-------------------------------------------------------------------------------
-- Title      : L1a LIne Decoder
-- Project    : RCU Trigger Receiver
-------------------------------------------------------------------------------
-- File             : $RCSfile: L1_line_decoder.vhd,v $
-- Last edited by   : $Author: alme $
-- Last update      : $Date: 2007/12/10 15:24:09 $
-- Current Revision : $Revision: 1.3 $
-------------------------------------------------------------------------------
-- Description: Decoding L0 and L1 triggers received on the L1a line.
-------------------------------------------------------------------------------
-- Revision History : 
-- http://www.ift.uib.no/kjekscgi-bin/viewcvs.cgi/vhdlcvs/trigger_receiver/
-------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Instrumentation and 
-- Electronics Section, Dep. of Physics and Technology
-- University of Bergen, Norway, 2005
-- This file has been written by Johan Alme
-- Johan.Alme@ift.uib.no
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this firmware for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
USE ieee.std_logic_unsigned.ALL;

entity L1_line_decoder is
  port(
    clk                 : in  std_logic;
    reset_n             : in  std_logic;
    L1Accept            : in  std_logic;
    L0_support          : in  std_logic;
    L0                  : out std_logic;
    L1a                 : out std_logic);
end L1_line_decoder;

architecture behave of L1_line_decoder is
  signal shiftreg: std_logic_vector(2 downto 0);
begin
  
  p_gen_triggers : process(clk, reset_n)
  begin
    if (reset_n = '0') then
	  L0  <= '0';
	  L1a <= '0';
	elsif rising_edge(clk) then
	  L0  <= '0';
	  L1a <= '0';
	  if (L0_support = '0') then --old version of software 
	    if (shiftreg = "010") then 
		  L1a <= '1';
		end if;
	  else
	    if (shiftreg = "010") then 
		  L0  <= '1';
		end if;
	    if (shiftreg = "011") then 
		  L1a <= '1';
		end if;
	  end if;
	end if;
  end process p_gen_triggers;
  
  p_shiftreg : process(clk)
  begin  
    if falling_edge(clk) then
      shiftreg(0)          <= L1Accept;
	    shiftreg(2 downto 1) <= shiftreg(1 downto 0);
    end if;	
  end process p_shiftreg; 
end behave;
