-------------------------------------------------------------------------------------
-- Title            : ALICE TPC RCU2 Channel B detector
-- Project          : ALICE TPC RCU2 TTC tester
-------------------------------------------------------------------------------------
-- File             : $HeadURL: https://svn.cern.ch/reps/TPCRCU2/trunk/FPGA/diff_manch/clk_div.vhd $
-- Last edited by   : $Author: avelure $
-- Author:          : Arild Velure
-- Last update      : $Date: 2013-07-18 16:38:05 +0200 (to, 18 jul 2013) $
-- Current Revision : $Revision: 27 $
-- Company          : University of Bergen
-- Created          : 2013-06-28
-- Platform         : Libero v11.1
-- Device           : Microsemi SmartFusion2
-- Standard         : VHDL'02
-------------------------------------------------------------------------------------
-- Description:     This module checks if channel B is detected ie. less than 23   
--                  consecutive ones on channel A, reverse engineered from the TTCrx code (module lch_b_detect). 
--                  This is not a direct implementation as the original module used 
--                  flipflop and gates for counting.
-------------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Microelectronics group,
-- Dep. of Physics and Technology, University of Bergen, Norway, 2010.
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this software for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity ch_b_det is

  port (
    reset     : in  std_logic;
    ch_a_data : in  std_logic;
    clk_40    : in  std_logic;
    ch_b_det  : out std_logic
  );

end;

architecture behavioral of ch_b_det is
  signal count : std_logic_vector(4 downto 0) := (others => '0');
  signal inndata : std_logic;
begin

  counter : process(clk_40, reset)
  begin
    if reset = '1' then
      ch_b_det <= '0';
    elsif rising_edge(clk_40) then
        if (inndata = '0') then
          ch_b_det <= '1';
          count <= (others => '0');
        else
          count <= count + 1;
          ch_b_det <= not(count(4) and not count(3) and count(2) and count(1)); -- 22 or 23
        end if;
    end if;
  end process;
  inndata <= ch_a_data and not reset;
end;


