-------------------------------------------------------------------------------------
-- Title            : ALICE TPC RCU2 clock divider and phase changer
-- Project          : ALICE TPC RCU2 TTC tester
-------------------------------------------------------------------------------------
-- File             : $HeadURL: https://svn.cern.ch/reps/TPCRCU2/trunk/FPGA/diff_manch/clk_div.vhd $
-- Last edited by   : $Author: avelure $
-- Author:          : Arild Velure
-- Last update      : $Date: 2013-07-18 16:38:05 +0200 (to, 18 jul 2013) $
-- Current Revision : $Revision: 27 $
-- Company          : University of Bergen
-- Created          : 2013-06-28
-- Platform         : Libero v11.1
-- Device           : Microsemi SmartFusion2
-- Standard         : VHDL'02
-------------------------------------------------------------------------------------
-- Description:     This module is a clock divider reverse engineered from  
--                  the TTCrx code (module CDiv). 
--                  Pulsing change_ph changes the phase of clk_40
--                  
-------------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Microelectronics group,
-- Dep. of Physics and Technology, University of Bergen, Norway, 2010.
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this software for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity clk_div is

  port (
    clk_80    : in  std_logic;
    change_ph : in  std_logic;
    clk_40    : out std_logic
  );

end;

architecture behavioral of clk_div is
  signal change_ph_1 : std_logic := '0';
  signal change_ph_2b : std_logic := '0';
  signal change_ph_2b_nr_1 : std_logic;
  signal change_ph_2b_nr_1_x_3b : std_logic;
  signal change_ph_3 : std_logic := '0';
  signal change_ph_3b : std_logic;
begin

  controller : process(clk_80)
  begin
      if rising_edge(clk_80) then
        change_ph_1 <= change_ph;
        change_ph_2b <= not change_ph_1;
        change_ph_3 <= change_ph_2b_nr_1_x_3b;
    end if;
  end process;
  change_ph_2b_nr_1 <= change_ph_2b nor change_ph_1;
  change_ph_2b_nr_1_x_3b <= change_ph_2b_nr_1 xor change_ph_3b;
  change_ph_3b <= not change_ph_3;
  clk_40 <= not change_ph_3b;
end;


