---------------------------------------------------------------------------------------------------
-- DESIGN RCU FIRM WARE V.2.0
-- FILE NAME   : CMD_DECODER.VHD
-- DESIGN TEAM : Luciano Musa, Changzhou Xiang , Attiq ur Rehman
-- DESCRIPTION : All control Signal Active High, Data RD/WR on rising edge,ASYNCHRONOUS RESET AND CLEAR
-- CLOCK DOMAIN: RCU CLOCK
-- PURPOSE     : 
--           
-- AUTHORS     : Changzhou.Xiang (CERN-PH/ED)
-- LAST UPDATE : 20.02.08
-------------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library WORK;
use work.rcu_addr_set.all;

entity CMD_DECODER is
    Port(
		rst				: in  STD_LOGIC;
		clk				: in  STD_LOGIC;	-- RCU CLOCK.
		en 				: in  STD_LOGIC; 
		we 				: in  STD_LOGIC;
		addr			: in  STD_LOGIC_VECTOR(4 downto 0);
		arm_syn			: out std_logic;	-- REset sampling clock upon next L0
		cfg_fec	 		: out std_logic;	-- RE-CONFIGURE FEC
		exe_seq			: out std_logic;	-- START EXECUTE INSTRUCTION SEQUENCER
		abrtseq 		: out std_logic;	-- ABORT THE EXECUTE INSTRUCTION SEQUENCER
		swt_trg			: out std_logic;	-- SOFTWARE TRIGGER
		clr_err_reg		: out std_logic;	-- CLEAR ERROR REGISTER
		clr_rdrx_reg	: out std_logic;	-- CLEAR RDY RX ERROR REGISTER
		clr_trg_cnt		: out std_logic		-- CLEAR TRIGGER COUNTERS
	);
end CMD_DECODER;

architecture A_CMDDECODER of CMD_DECODER is 
	signal cmd_vector	: std_logic_vector(7 downto 0);
begin  
	cfg_fec	 		<= cmd_vector(0);	
	exe_seq			<= cmd_vector(1); 
	abrtseq 		<= cmd_vector(2); 
	swt_trg			<= cmd_vector(3); 
	clr_err_reg		<= cmd_vector(4); 
	clr_rdrx_reg	<= cmd_vector(5); 
	clr_trg_cnt		<= cmd_vector(6);
	arm_syn     	<= cmd_vector(7);
				
	process(rst,clk)
	begin
		if rst='1' then
			cmd_vector <= (others=>'0'); 
		elsif clk' event and clk='1' then
			if en='1' and we='1' then
				case addr is						
					when "00011" =>
						cmd_vector <= "00000001";
						
					when "00100" =>
						cmd_vector <= "00000010";

					when "00101" =>
						cmd_vector <= "00000100";

					when "00110" =>
						cmd_vector <= "00001000";
						
					when "00111" =>
						cmd_vector <= "00010000";
						
					when "01000" =>
						cmd_vector <= "00100000";
						
					when "01001" =>
						cmd_vector <= "01000000";
					when "01010" =>
						cmd_vector <= "10000000";
						
					when others =>
						cmd_vector <= (others=>'0');
				end case;
			else
				cmd_vector <= (others=>'0');
			end if;
		end if;
	end process;
			
end;

