-------------------------------------------------------------------------------
-- Title      : DCS interface Package
-- Project    : RCU DCS interface
-------------------------------------------------------------------------------
-- File             : $RCSfile: dcs_interface_pkg.vhd,v $
-- Last edited by   : $Author: alme $
-- Last update      : $Date: 2008/02/15 12:23:43 $
-- Current Revision : $Revision: 1.4 $
-------------------------------------------------------------------------------
-- Description      : Constant and function library for RCU Trigger Receiver
--                    design  
-------------------------------------------------------------------------------
-- Revision History : 
-- http://portal1.ift.uib.no/cgi-bin/viewcvs.cgi/vhdlcvs/rcu_cpld/
-------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Instrumentation and 
-- Electronics Section, Dep. of Physics and Technology
-- University of Bergen, Norway, 2005
-- This file has been written by Johan Alme
-- Johan.Alme@ift.uib.no
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this software for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

package dcs_interface_pkg is
 
 -- address information (this should be moved to a global adress mapping definition file.)
  -- memory spaces.
  -- Safety module address. Always ack these.
  constant c_MSM_space          : std_logic_vector(3 downto 0):=X"8";
  -- Actel space should NEVER be acked
  constant c_Actel_space        : std_logic_vector(3 downto 0):=X"B";
  -- treat as normal except for the subadresses 0xA00 and 0xA01 that should not be acked.
  constant c_trigger_space      : std_logic_vector(3 downto 0):=X"4"; 
  -- sub adresses
  -- belongs to trigger space. The two addresses are defined on the DCS board and should not be acked. 
  constant c_dcsSetBunchReset   : std_logic_vector(11 downto 0):= X"A00";
  constant c_dcsSetEventReset   : std_logic_vector(11 downto 0):= X"A01";
  
  constant c_global_reset       : std_logic_vector(15 downto 0):= X"5300";
  constant c_fec_reset          : std_logic_vector(15 downto 0):= X"5301";
  constant c_rcu_reset          : std_logic_vector(15 downto 0):= X"5302";
  constant c_arbiter_irq        : std_logic_vector(15 downto 0):= X"5310"; -- interrupts SIU grant
  constant c_grant              : std_logic_vector(15 downto 0):= X"5311"; -- grant information given
  
  --Constant defining mem mapped mode on which the interface should be active
  constant c_memMappedMode0      : std_logic_vector(1 downto 0) := "00";
  constant c_memMappedMode1      : std_logic_vector(1 downto 0) := "11";
       
end package dcs_interface_pkg;
