-------------------------------------------------------------------------------------
-- Title            : ALICE TPC RCU2 Channel demultiplexer
-- Project          : ALICE TPC RCU2 TTC tester
-------------------------------------------------------------------------------------
-- File             : $HeadURL: https://svn.cern.ch/reps/TPCRCU2/trunk/FPGA/diff_manch/clk_div.vhd $
-- Last edited by   : $Author: avelure $
-- Author:          : Arild Velure
-- Last update      : $Date: 2013-07-18 16:38:05 +0200 (to, 18 jul 2013) $
-- Current Revision : $Revision: 27 $
-- Company          : University of Bergen
-- Created          : 2013-06-28
-- Platform         : Libero v11.1
-- Device           : Microsemi SmartFusion2
-- Standard         : VHDL'02
-------------------------------------------------------------------------------------
-- Description:     This module demultiplexes channel A and B data, reverse  
--                  engineered from the TTCrx code (module lDemultiplexAB). 
--                  channel_b_detect has been moved up one level, clk_trigger output removed
--                  
-------------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Microelectronics group,
-- Dep. of Physics and Technology, University of Bergen, Norway, 2010.
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this software for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity demultiplexAB is

  port (
    rst       : in  std_logic;
    clk_40    : in  std_logic;
    data_80   : in std_logic;
    ch_a      : out std_logic;
    ch_b      : out std_logic
  );

end;

architecture behavioral of demultiplexAB is
  signal data_80_delay : std_logic;
begin

  output : process(clk_40, rst)
  begin
    if (rst = '1') then
      ch_a <= '0';
      ch_b <= '0';
    elsif rising_edge(clk_40) then
      ch_a <= data_80;
      ch_b <= data_80_delay;
    end if;
  end process;
  
  data_delay : process(clk_40, rst)
  begin
    if (rst = '1') then
      data_80_delay <= '0';
    elsif falling_edge(clk_40) then
      data_80_delay <= data_80;
    end if;
  end process;
end;


