---------------------------------------------------------------------------------------------------
-- DESIGN RCU FIRM WARE V.2.0
-- FILE NAME   : MUX_ARBIT.V
-- DESIGN TEAM : Luciano Musa, Changzhou Xiang , Attiq ur Rehman
-- DESCRIPTION : All control Signal Active High, Data RD/WR on rising edge,ASYNCHRONOUS RESET AND CLEAR
-- CLOCK DOMAIN: RCU CLOCK
-- PURPOSE     : MUX WITH SIU AND DCS DATA BUS
--           
-- AUTHORS     : Attiq ur Rehman (CERN-PH/ED)
-- LAST UPDATE : 01.02.08
-------------------------------------------------------------------------------------------------------



library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity MUX_ARBIT is
	Port ( 
		--input----------------
		
		dcs_din,siu_din  	: in  STD_LOGIC_VECTOR (31 downto 0); -- data input from siuy and dcs
		dcs_add, siu_add 	: in  STD_LOGIC_VECTOR (15 downto 0); 				
		dcs_we, siu_we  	: in STD_LOGIC; 
		rcu_dout 			: in  STD_LOGIC_VECTOR (31 downto 0); -- data output from rcu as a result of read transacton
		
		dcs_dout, siu_dout  : out  STD_LOGIC_VECTOR (31 downto 0); -- rcu_dout whihc will be delivered to either siu or dcs
		rcu_add				: out  STD_LOGIC_VECTOR (15 downto 0); 
		rcu_data		   	: out  STD_LOGIC_VECTOR (31 downto 0); -- rcy data bus -- into rcu
		rcu_we				: out  STD_LOGIC;

		siu_bg, dcs_bg		:in std_logic
		
	);				
end MUX_ARBIT;

architecture A_MUXARBIT of MUX_ARBIT is
		
		
		signal sel : STD_LOGIC_VECTOR (1 downto 0);
		
begin
		sel <= (siu_bg & dcs_bg);
		
		process(sel,dcs_din,dcs_add, rcu_dout,dcs_we ,siu_din ,siu_add ,siu_we )
		
		begin
				case sel is

				when "01" => 	rcu_data <=dcs_din; 
								rcu_add <= dcs_add;  
								rcu_we <= dcs_we; 
								dcs_dout <= rcu_dout; 
								siu_dout <= (others=>'0');
								
				when "10" => 	rcu_data <=siu_din; 
								rcu_add <= siu_add;  
								rcu_we <= siu_we; 
								siu_dout <= rcu_dout; 
								dcs_dout <= (others=>'0');
								
				when others =>  				
				
				end case;				
		end process;
		
		
end;

