-------------------------------------------------------------------------------
-- DESIGN RCU FIRM WARE V.2.0
-- FILE NAME   : RCU_ADDR_SET.VHD
-- DESIGN TEAM : Luciano Musa, Changzhou Xiang , Attiq ur Rehman
-- DESCRIPTION : RCU ADDRESS DECODER

-------------------------------------------------------------------------------


library IEEE;				
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.numeric_bit.all;

package   rcu_addr_set is
	--------------- CONFIGURATION REGISTERS --------------- 
	constant 	fecactlist 		: std_logic_vector(7 downto 0) := X"00"; --x 00; 
	constant 	altroif 		: std_logic_vector(7 downto 0) := X"01"; --x 01;
	constant 	trgconf 		: std_logic_vector(7 downto 0) := X"02"; --x 02;
	constant 	rdomod 			: std_logic_vector(7 downto 0) := X"03"; --x 03;
	constant 	rdocfg1 		: std_logic_vector(7 downto 0) := X"04"; --x 04;
	constant 	rdocfg2 		: std_logic_vector(7 downto 0) := X"05"; --x 05;
	constant 	rcuversion 		: std_logic_vector(7 downto 0) := X"06"; --x 06;
	constant 	bpversion 		: std_logic_vector(7 downto 0) := X"07"; --x 07;
	constant 	rcuid	 		: std_logic_vector(7 downto 0) := X"08"; --x 08;
	constant 	fecrdolist 		: std_logic_vector(7 downto 0) := X"09"; --x 09; 
	
	--------------- STATUS & ERROR REGISTERS ---------------
	constant   	fecerra			: std_logic_vector(7 downto 0) := X"10"; --x 10;
	constant 	fecerrb			: std_logic_vector(7 downto 0) := X"11"; --x 11;
	constant 	rdoerr			: std_logic_vector(7 downto 0) := X"12"; --x 12;
--	constant 	rcubus			: std_logic_vector(7 downto 0) := X"13"; --x 13;	-- Defined already in DCS_interface module
	constant 	busstat 		: std_logic_vector(7 downto 0) := X"14"; --x 14;
	constant 	bussttrsf 		: std_logic_vector(7 downto 0) := X"15"; --x 15;
	constant 	rcubusbsy		: std_logic_vector(7 downto 0) := X"16"; --x 16;
	
	constant 	mebstcnt 		: std_logic_vector(7 downto 0) := X"1b"; --x 1b;
	constant 	swttrgcnt 		: std_logic_vector(7 downto 0) := X"1c"; --x 1c;
	constant 	auxtrgcnt 		: std_logic_vector(7 downto 0) := X"1d"; --x 1d;
	constant 	ttcl2acnt 		: std_logic_vector(7 downto 0) := X"1e"; --x 1e;
	constant 	ttcl2rcnt 		: std_logic_vector(7 downto 0) := X"1f"; --x 1f;
	constant 	ttcl1rcnt 		: std_logic_vector(7 downto 0) := X"31"; --x 1f;
	
	constant 	dstbacnt 		: std_logic_vector(7 downto 0) := X"20"; --x 20;
	constant 	dstbbcnt 		: std_logic_vector(7 downto 0) := X"21"; --x 21;
	constant 	trsfacnt 		: std_logic_vector(7 downto 0) := X"22"; --x 22;
	constant 	trsfbcnt 		: std_logic_vector(7 downto 0) := X"23"; --x 23;
	constant 	ackacnt 		: std_logic_vector(7 downto 0) := X"24"; --x 24;
	constant 	ackbcnt 		: std_logic_vector(7 downto 0) := X"25"; --x 25;
	constant 	cstbacnt 		: std_logic_vector(7 downto 0) := X"26"; --x 26;
	constant 	cstbbcnt 		: std_logic_vector(7 downto 0) := X"27"; --x 27;
	constant 	dstbnuma 		: std_logic_vector(7 downto 0) := X"28"; --x 28;
	constant 	dstbnumb 		: std_logic_vector(7 downto 0) := X"29"; --x 29;
	constant 	chadderrcnt		: std_logic_vector(7 downto 0) := X"2a"; --x 2a;
	constant 	blerrcnt 		: std_logic_vector(7 downto 0) := X"2b"; --x 2b;
	
	constant 	abdfsm_st	    : std_logic_vector(7 downto 0) := X"2c"; --x 2c;
	constant 	rdofsm_st	    : std_logic_vector(7 downto 0) := X"2d"; --x 2d;
	constant 	insseq_st	    : std_logic_vector(7 downto 0) := X"2e"; --x 2e;
	constant 	evmng_st	    : std_logic_vector(7 downto 0) := X"2f"; --x 2f;
	constant 	da_st	    	: std_logic_vector(7 downto 0) := X"30"; --x 30;
	
	----------------- SLOW CONTROL ADDRESS RANGE FOR DATA AND ADDRESS-----------------------------
	-- dont like te address space allocation but have to keep it to avoid the modification of software

	constant BC_ADDR   		    	: std_logic_vector(3 downto 0) := x"5"; -- FEC ADDR, BC REG ADDR - TYPE OF TRANSACTION RD/WR
	constant BC_DATA       			: std_logic_vector(3 downto 0) := x"6"; -- WRITE DATA FOR WRITE TRANSACTION ON BC
	constant BC_RSLT	        	: std_logic_vector(3 downto 0) := x"2"; -- RESULT OF READ
	constant INT_EN				    : std_logic_vector(3 downto 0) := x"4"; -- REGISTER TO ENABLE THE INTERRUPT
	constant old_act_fec     		: std_logic_vector(3 downto 0) := x"0"; 
	constant old_fw_ver      		: std_logic_vector(3 downto 0) := x"8"; 
	constant BC_EROR            	: std_logic_vector(3 downto 0) := x"9";
	constant no_St_wrds            	: std_logic_vector(3 downto 0) := x"9";
	constant DBG_1		       	 	: std_logic_vector(3 downto 0) := x"1";	
	constant MSK_ERR        		: std_logic_vector(3 downto 0) := x"7";
	constant sc_st_reg	        	: std_logic_vector(3 downto 0) := x"3";
	constant BC_EXEC      			: std_logic_vector(3 downto 0) := x"0";
	constant RESTOR_ACT   			: std_logic_vector(3 downto 0) := x"3";
	constant RESTOR_RDO   			: std_logic_vector(3 downto 0) := x"4";
	constant RST_RSLT_REG  			    : std_logic_vector(3 downto 0) := x"2";
	constant RST_ERR_REG  			    : std_logic_vector(3 downto 0) := x"1";
	constant RST_MEM_REG  			    : std_logic_vector(3 downto 0) := x"5";
	constant MSM_REG      			: std_logic_vector(11 downto 0) := x"800";
	constant MSM_CMD      			: std_logic_vector(11 downto 0) := x"801";
	constant MSM_MEM_A      		: std_logic_vector(11 downto 0) := x"810"; ---- X 8100 - 810f
	constant MSM_MEM_B      		: std_logic_vector(11 downto 0) := x"811"; ---- X 8110 - 811f
	--------------- BLOCK MEMORY ---------------
	constant addr_im			    : std_logic_vector(15 downto 12) := "0000";			--x 0000-0FFF
	
	constant addr_rolm			    : std_logic_vector(15 downto 12) := "0001"; 		--x 1000-1FFF
	
	constant addr_rm	 			: std_logic_vector(15 downto 13) := "001"; 			--x 2000-3FFF
	
	constant addr_ttcrx 			: std_logic_vector(15 downto 12) := "0100"; 		--x 4000-4FFF
	
	constant addr_hlm			    : std_logic_vector(15 downto  7) := "010100000";	--x 5000-507F
	
	constant addr_reg			    : std_logic_vector(15 downto  8) := "01010001"; 	--x 5100-51FF
	constant addr_cmd				: std_logic_vector(15 downto  5) := "01010011000"; 	--x 5300-531F
	
	constant addr_dm_0 				: std_logic_vector(15 downto  8) := "01011000"; 	--x 5800-58FF;
	constant addr_dm_1 				: std_logic_vector(15 downto  8) := "01011001"; 	--x 5900-59FF;
	constant addr_dm_2 				: std_logic_vector(15 downto  8) := "01011010"; 	--x 5A00-5AFF;
	constant addr_dm_3 				: std_logic_vector(15 downto  8) := "01011011"; 	--x 5B00-5BFF;
	constant addr_dm_4 				: std_logic_vector(15 downto  8) := "01011100"; 	--x 5C00-5CFF;
	constant addr_dm_5 				: std_logic_vector(15 downto  8) := "01011101"; 	--x 5D00-5DFF;
	constant addr_dm_6 				: std_logic_vector(15 downto  8) := "01011110"; 	--x 5E00-5EFF;
	constant addr_dm_7 				: std_logic_vector(15 downto  8) := "01011111"; 	--x 5F00-5FFF;
	
	constant addr_dm 				: std_logic_vector(15 downto 11) := "01100"; 		--x 5800-5FFF;
	
end rcu_addr_set;
