---------------------------------------------------------------------------------------------------
-- DESIGN RCU FIRM WARE V.2.0
-- FILE NAME   : RCU_DEC.V
-- DESIGN TEAM : Luciano Musa, Changzhou Xiang , Attiq ur Rehman
-- DESCRIPTION : All control Signal Active High, Data RD/WR on rising edge,ASYNCHRONOUS RESET AND CLEAR
-- CLOCK DOMAIN: RCU CLOCK
-- PURPOSE     : RCU INTERFACE WITH SIU AND DCS.
--           
-- last update   : Changzhou.Xiang (CERN-PH/ED)
--
-- UPDATE TIME : 21.07.08
-- NOTE        : Add FSM status of DA.
--
-- UPDATE TIME : 12.06.08
-- NOTE        : Add "inst_err_chk" as ouput, instruction for TPC/PHOS/FMD.
---- UPDATE TIME : 10.06.08
-- NOTE        : 		FEC_AL_fsc 	we_FEC_AL_fsc added to write the fec active list from safty module.
-- FSM WR and FSM EVENLENGTH state machine STATES are Registered:- dated 08/06/08
-- Lower Hierarchy file change is "RCU_REG.vhd
-- line number 338,339,349,350,447,448. ins_by was used to disabel to the bus request of DCS/SIU. this feature is removed.
--
-- UPDATE TIME : 29.05.08
-- NOTE        : Add "da_bsy" as input.
--
-- UPDATE TIME : 28.05.08
-- NOTE        : Add checking RDYRX status register.
--             : Change scel_err_reg width from 8 to 6.
--             : Change dstb/trsf counters bit width.
--
-- UPDATE TIME : 26.05.08
-- NOTE        : Add is_fail_reg to show getting failed command in instruction memory.
--
-- UPDATE TIME : 23.05.08
-- NOTE        : Modify RCU bus busy register from 4 bits to 1bit.
--
-- UPDATE TIME : 19.05.08
-- NOTE        : Add meb status counter to registers
--
-- UPDATE TIME : 18.05.08
-- NOTE        : Add RCU bus busy status register.
--
-- UPDATE TIME : 16.05.08
-- NOTE        : Add RCU ID register.
--
-- UPDATE TIME : 15.05.08
-- NOTE        : Change dstb/trsf/ack/cstb counters bit width.
--
-- UPDATE TIME : 12.05.08
-- NOTE        : Use signal "clr_err_reg" of "CMD_DECODER" clear ALTRO bus signal counters.
--
-- UPDATE TIME : 11.05.08
-- NOTE        : Add enable/disable configuration bit for check RDYRX status in register RDOMOD[3].
--               Default value is 0, enable checking. When set 1, disable checking.
--               Add ALTRO bus signal(dstb/trsf/cstb/ack) counters.
--               Add block length of last written data memory.
-------------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity RCU_DEC is
	port(		
		clk 				: in std_logic;
		rst_n 				: in std_logic;
		rcu_wen				: out std_logic;	-- RCU WRITE ENABLE
		rcu_add				: out std_logic_vector(15 downto 0);	-- RCU BUS ADDRESS
		rcu_data_out		: out std_logic_vector(31 downto 0);	-- DATA TO RCU BUS
		
		----------- DCS INTERFACE -----------
		dcs_we              : in   std_logic;                     -- write enable to the internal modules
		dcs_busBadd         : in   std_logic_vector(15 downto 0); -- address passed through
		dcs_busBdata_in     : out  std_logic_vector(31 downto 0); -- data to internal modules from DCS
		dcs_busBdata_out    : in  std_logic_vector(31 downto 0); -- data to internal modules from DCS
		
		dcs_bi              : in   std_logic;                     -- interrupt in case DCS needs bus
		dcs_bg              : out  std_logic;                     -- dcs bus grant from arbiter
		dcs_br              : in  std_logic;                     -- dcs bus request to arbiter
		
		altro_err_a			: in  STD_LOGIC;
		altro_err_b			: in  STD_LOGIC;
		
		
		----------- SIU INTERFACE -----------			
		siu_bi   		 	: in    std_logic;  -- Bus interrupt
		siu_br   		 	: in    std_logic;  -- Bus request
		siu_bg   		 	: out   std_logic;  -- Bus grant
		siu_data_in   		: out	std_logic_vector (31 downto 0);		-- DATA TO SIU_INTERFACE.
		siu_data_out  		: in   std_logic_vector (31 downto 0);		-- DATA FROM SIU_INTERFACE.
		
		siu_add  		 	: in    std_logic_vector (15 downto 0); 	-- SIU ADDRESS FROM SIU_INTERFACE
		siu_we   		 	: in    std_logic;	-- SIU WRITE ENABLE, FROM SIU_INTERFACE
			
		----------- MEMORY DATA OUT TO RCU/SIU/DCS -----------
		im_dout				: in  std_logic_vector(21 downto 0);		-- INSTRUCTION SEQUENCE MEMORY DATA OUT, TO RCU BUS DATA_OUT
		rcu_rolm_out		: in  std_logic_vector(11 downto 0);		-- READOUT LIST MEMORY DATA OUT, TO RCU BUS DATA_OUT
		rm_dout				: in  std_logic_vector(24 downto 0);		-- RESULT MEMORY DATA OUT, TO RCU BUS DATA_OUT
		rcu_ttcrx_dout		: in  std_logic_vector(31 downto 0);		-- TRIGGER_RECEIVER MEMORY DATA OUT, TO RCU BUS DATA_OUT
		rcu_hlm_out			: in  std_logic_vector(31 downto 0);		-- HIT LIST MEMORY DATA OUT, TO RCU BUS DATA_OUT
		rcu_dm_out			: in  std_logic_vector(27 downto 0);		-- CHANNEL DATA MEMORY DATA OUT, TO RCU BUS DATA_OUT
		
		----------- MEMORY ENABLE SIGNALS -----------		
		rcu_im_en			: out std_logic;	-- ENABLE INSTRUCTION MEMORY
		rcu_rolm_en			: out std_logic;	-- ENABLE READOUTLIST MEMORY
		rcu_rm_en			: out std_logic;	-- ENABLE RESULT MEMORY
		rcu_hlm_en 			: out std_logic;    -- ENABLE HIT LIST MEMORY
		sel 		   		: out std_logic_vector(7 downto 0); 		-- CHANNEL DATA MEMORIES SELECTION

		----------- CHECK BUS -----------
		ins_bsy 			: in  std_logic;
		is_fail_reg			: in  std_logic;	-- Show getting failed command in instruction memory.
		exe_ev			   	: in  std_logic;	-- Execute event, to clear event readout counters.
		----------- COMMANDS -----------		
		cfg_fec	 			: out std_logic;	-- RE-CONFIGURE FEC
		exe_seq				: out std_logic;	-- START INSTRUCTION SEQUENCE
		abrtseq				: out std_logic;	-- ABORT INSTRUCTION SEQUENCE
		swt_trg				: out std_logic;	-- SOFTWARE TRIGGER
		clr_err_reg			: out std_logic;	-- CLEAR ERROR REGISTER
		clr_trg_cnt			: out std_logic;	-- CLEAR TRIGGER COUNTER
		arm_syn				: out std_logic;	-- REset sampling clock upon next L0
		
		-------------------------------------------------------------------------------
		-- debug for busy box & MEB
		meb_full 			: in  STD_LOGIC;
	--	st_wrd_cnt 			: IN   std_logic_vector (19 downto 0);
		----------- CONFIGURATION AND STATUS REGISTERS ----------- 	
		fecerr_a			: in  STD_LOGIC_VECTOR(31 downto 0);		-- FEC ERROR REGISTER FOR BRANCH A
		fecerr_b	      	: in  STD_LOGIC_VECTOR(31 downto 0);		-- FEC ERROR REGISTER FOR BRANCH B
		
		meb_st_cnt			: in  STD_LOGIC_VECTOR(3 downto 0);		-- MEB STATUS COUNTER
		swt_trg_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);		-- NUMBER OF VALID SOFTWARE TRIGGER
		aux_trg_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);		-- NUMBER OF VALID HARDWARE TRIGGER
		ttc_L2a_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);		-- NUMBER OF VALID L2A TRIGGER
		ttc_L2r_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);		-- NUMBER OF VALID L2R TRIGGER
		ttc_L1r_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);		-- NUMBER OF VALID L2R TRIGGER
		err_code 			: in  std_logic_vector(2 downto 0); 	-- SCAN EVENT LENGTH ERROR CODE
		err_scev 			: in  std_logic;	-- 1 BIT ERROR DURING SCANEVLENGTH
		da_bsy	 			: in  std_logic;	-- DA IS BUSY
		rd_rx				: in  STD_LOGIC;	-- RDYRX from SIU interface
		en_blerr_cnt 		: in  STD_LOGIC;	-- ENABLE BLOCK LENGTH ERROR COUNTER
		en_chadderr_cnt 	: in  STD_LOGIC;	-- ENABLE COUNT CHANNEL ADDRESS ERROR COUNTER
		
		data_ready		: in  STD_LOGIC_VECTOR(3 downto 0);	-- status of data memories from AIM
		dm_sel			: in  STD_LOGIC_VECTOR(1 downto 0) ; -- from data assembler
		

		prg_dly				: out STD_LOGIC_VECTOR( 1 downto 0);		-- CSTB PROGRAMABLE DELAY
		trg_prg_dly			: out STD_LOGIC_VECTOR(12 downto 0);		-- L2 DELAY AFTER L1 IN TEST TRIGGER MODE
		en_chkrdyrx_n			: out STD_LOGIC;	-- ENABLE/DISABLE CHECK RDYRX
		sprs_rdo	  			: out STD_LOGIC; 	-- ENABLE SPARSE READOUT
		skp_emp_ch				: out STD_LOGIC;
		trg_source			: out STD_LOGIC_VECTOR( 2 downto 0);        -- TRIGGER SOURCE
		trg_mode  			: out STD_LOGIC;	-- TRIGGER MODE
		meb_mode	   		: out STD_LOGIC;	-- MEB MODE
		es_on					: out STD_LOGIC;	-- EXCUTE SEQUENCER ON SOE/EOE
		samp_frec			: out STD_LOGIC_VECTOR(3 downto 0);		-- SAMPLE FREQUENCE.
		rcu_version 		: out STD_LOGIC_VECTOR(31 downto 0);	-- RCU version
		rcu_id 				: out STD_LOGIC_VECTOR( 8 downto 0);	-- RCU ID
		bp_version  		: out STD_LOGIC;	-- BP version
		rdo_cfg1			: out STD_LOGIC_VECTOR(19 downto 0);
		rdo_cfg2			: out STD_LOGIC_VECTOR(24 downto 5);
		rdo_err				: out STD_LOGIC_VECTOR(8 downto 0);		-- READOUT ERROR, TO DA
		bl_err_cnt			: out STD_LOGIC_VECTOR(11 downto 0);		-- Block length mismatch counter, TO DA
		chadd_err_cnt		: out STD_LOGIC_VECTOR(11 downto 0);		-- Channel address mismatch counter, TO DA
		
--  safty module write signals for the active fron end card list

		FEC_AL_fsc : IN std_logic_vector(31 downto 0);
		we_FEC_AL_fsc : IN std_logic;
		
-- mask the check on block length mismatch and channel addres mismatch.	
	
		mask_ch_add : out STD_LOGIC;
		mask_blk_len : out STD_LOGIC;
		
		
		-- ALTRO BUS STATUS
		dstb_a 				: in  STD_LOGIC;
		cstb_a 				: in  STD_LOGIC;
		ack_a 				: in  STD_LOGIC;
		trsf_a 				: in  STD_LOGIC;
		dstb_b 				: in  STD_LOGIC;
		cstb_b 				: in  STD_LOGIC;
		ack_b 				: in  STD_LOGIC;
		trsf_b 				: in  STD_LOGIC;
		L1 					: in  STD_LOGIC;
		L2 					: in  STD_LOGIC; 
		
		dstb_numA	: in  std_logic_vector(8 downto 0);		-- DSTB NUMBER OF LAST BLOCK FOR BRANCH A
		dstb_numB	: in  std_logic_vector(8 downto 0);		-- DSTB NUMBER OF LAST BLOCK FOR BRANCH B
		
		fec_act_list		: out STD_LOGIC_VECTOR(31 downto 0);		-- ACTIVE FEC LIST
		inst_err_chk		: out STD_LOGIC_VECTOR( 1 downto 0)	;		-- Instruction error checking selection.
		fsmwr_st_a			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmrd_st_a			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmwr_st_b			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmrd_st_b			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmevrdoctrl_st		: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmselbr_st_a		: in  STD_LOGIC_VECTOR(1 downto 0);		-- FSM state,from AIM 
		fsmselbr_st_b		: in  STD_LOGIC_VECTOR(1 downto 0);		-- FSM state,from AIM 
		fsmevlen_st			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmscevlen_st		: in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from AIM 
		fsmrdo_st_a			: in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from AIM 
		fsmrdo_st_b			: in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from AIM 
		fsminstseq_st		: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from IS
		fsminstloop_st		: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from IS 
		fsmtsttrg_st		: in  STD_LOGIC_VECTOR(1 downto 0);		-- FSM state,from EV_MNG
		fsmttctrg_st		: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from EV_MNG
		fsmev_st			: in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from EV_MNG
		fsminsseq_st		: in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from EV_MNG 	
		fsmru_st			: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from RU 	
		fsmdamain_st		: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrdcdh_st			: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrdcdhsoft_st		: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrddm_st			: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrdpar_st			: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrdrm_st			: in  STD_LOGIC_VECTOR(3 downto 0)		-- FSM state,from DA 
		
	);

end RCU_DEC;

architecture A_RCUDEC of RCU_DEC is

	COMPONENT MUX_ARBIT
	PORT(
		dcs_din : IN std_logic_vector(31 downto 0);
		siu_din : IN std_logic_vector(31 downto 0);
		dcs_add : IN std_logic_vector(15 downto 0);
		siu_add : IN std_logic_vector(15 downto 0);
		dcs_we : IN std_logic;
		siu_we : IN std_logic;
		rcu_dout : IN std_logic_vector(31 downto 0);
		siu_bg : IN std_logic;
		dcs_bg : IN std_logic;          
		dcs_dout : OUT std_logic_vector(31 downto 0);
		siu_dout : OUT std_logic_vector(31 downto 0);
		rcu_add : OUT std_logic_vector(15 downto 0);
		rcu_data : OUT std_logic_vector(31 downto 0);
		rcu_we : OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT ARBIT
	PORT(
		siu_br : IN std_logic;
		siu_bi : IN std_logic;
		dcs_br : IN std_logic;
		dcs_bi : IN std_logic;
		clk : IN std_logic;
		rst : IN std_logic;          
		siu_bg : OUT std_logic;
		dcs_bg : OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT ADDR_DECODER
	PORT(
		rcu_add : IN std_logic_vector(15 downto 5);          
		rcu_im_en : OUT std_logic;
		rcu_rolm_en : OUT std_logic;
		rcu_rm_en : OUT std_logic;
		rcu_ttcrx_en : OUT std_logic;
		rcu_hlm_en : OUT std_logic;
		rcu_reg_en : OUT std_logic;
		rcu_cmd_en : OUT std_logic;
		sel : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;
	
	COMPONENT CMD_DECODER
	PORT(
		rst : IN std_logic;
		clk : IN std_logic;
		en : IN std_logic;
		we : IN std_logic;
		addr : IN std_logic_vector(4 downto 0);          
		cfg_fec : OUT std_logic;
		exe_seq : OUT std_logic;
		abrtseq : OUT std_logic;
		swt_trg : OUT std_logic;
		arm_syn			: out std_logic;	-- REset sampling clock upon next L0 
		clr_err_reg : OUT std_logic;
		clr_rdrx_reg : OUT std_logic;
		clr_trg_cnt : OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT RCU_REG
	PORT(

		en : IN std_logic;
		we : IN std_logic;
		addr : IN std_logic_vector(7 downto 0);
		din : IN std_logic_vector(31 downto 0);
		data_ready		: in  STD_LOGIC_VECTOR(3 downto 0);	-- status of data memories from AIM
		dm_sel			: in  STD_LOGIC_VECTOR(1 downto 0) ; -- from data assembler
		fecerr_a : IN std_logic_vector(31 downto 0);
		fecerr_b : IN std_logic_vector(31 downto 0);
		da_bsy : IN std_logic;
		rd_rx : IN std_logic;
		-- mask the check on block length mismatch and channel addres mismatch.
		meb_full 		: in  STD_LOGIC;
		mask_ch_add : out STD_LOGIC;
		mask_blk_len : out STD_LOGIC;
		clr_rdrx_reg : IN std_logic;
		scel_err_reg : IN std_logic_vector(5 downto 0);
		blk_err_reg : IN std_logic;
		add_err_reg : IN std_logic;
		rcu_busbsy : IN std_logic_vector(1 downto 0);
		meb_st_cnt : IN std_logic_vector(3 downto 0);
		swt_trg_cnt : IN std_logic_vector(19 downto 0);
		aux_trg_cnt : IN std_logic_vector(19 downto 0);
		ttc_L2a_cnt : IN std_logic_vector(19 downto 0);
		ttc_L2r_cnt : IN std_logic_vector(19 downto 0);
		ttc_L1r_cnt	: in  STD_LOGIC_VECTOR(19 downto 0);		-- NUMBER OF VALID L2R TRIGGER
		cstb_a : IN std_logic;
		ack_a : IN std_logic;
		trsf_a : IN std_logic;
		cstb_b : IN std_logic;
		ack_b : IN std_logic;
		trsf_b : IN std_logic;
		L1 : IN std_logic;
		L2 : IN std_logic;
		altro_err_a			: in  STD_LOGIC;
		altro_err_b			: in  STD_LOGIC;
		dstba_cnt : IN std_logic_vector(19 downto 0);
		dstbb_cnt : IN std_logic_vector(19 downto 0);
		trsfa_cnt : IN std_logic_vector(11 downto 0);
		trsfb_cnt : IN std_logic_vector(11 downto 0);
		acka_cnt : IN std_logic_vector(11 downto 0);
		ackb_cnt : IN std_logic_vector(11 downto 0);
		cstba_cnt : IN std_logic_vector(11 downto 0);
		cstbb_cnt : IN std_logic_vector(11 downto 0);
		dstb_numA : IN std_logic_vector(8 downto 0);
		dstb_numB : IN std_logic_vector(8 downto 0);
		bl_err_cnt : IN std_logic_vector(11 downto 0);
		chadd_err_cnt : IN std_logic_vector(11 downto 0);
		FEC_AL_fsc : IN std_logic_vector(31 downto 0);
		we_FEC_AL_fsc : IN std_logic;
		clk : IN std_logic;
		rst : IN std_logic;          
		dout : OUT std_logic_vector(31 downto 0);
		fec_act_list : OUT std_logic_vector(31 downto 0);
		inst_err_chk : OUT std_logic_vector( 1 downto 0);
		prg_dly : OUT std_logic_vector(1 downto 0);
		samp_frec : OUT std_logic_vector(3 downto 0);
		trg_source : OUT std_logic_vector(2 downto 0);
		trg_mode : OUT std_logic;
		trg_prg_dly : OUT std_logic_vector(12 downto 0);
		en_chkrdyrx_n : OUT std_logic;
		sprs_rdo : OUT std_logic;
		skp_emp_ch				: out STD_LOGIC;
		es_on : OUT std_logic;
		meb_mode : OUT std_logic;
		rdo_cfg1 : OUT std_logic_vector(19 downto 0);
		rdo_cfg2 : OUT std_logic_vector(24 downto 5);
		rcu_version : OUT std_logic_vector(31 downto 0);
		rcu_id : OUT std_logic_vector(8 downto 0);
		bp_version : OUT std_logic;
		rdrx_erreg : OUT std_logic;		
		rdo_err : OUT std_logic_vector(8 downto 0);
		fsmwr_st_a : in  STD_LOGIC_VECTOR(3 downto 0);	
		fsmrd_st_a : in  STD_LOGIC_VECTOR(3 downto 0);	
		fsmwr_st_b : in  STD_LOGIC_VECTOR(3 downto 0);	
		fsmrd_st_b : in  STD_LOGIC_VECTOR(3 downto 0);	
		fsmevrdoctrl_st : in  STD_LOGIC_VECTOR(3 downto 0);	
		fsmselbr_st_a : in  STD_LOGIC_VECTOR(1 downto 0);	
		fsmselbr_st_b : in  STD_LOGIC_VECTOR(1 downto 0);	
		fsmevlen_st : in  STD_LOGIC_VECTOR(3 downto 0);	
		fsmscevlen_st : in  STD_LOGIC_VECTOR(2 downto 0);	
		fsmrdo_st_a : in  STD_LOGIC_VECTOR(2 downto 0);	
		fsmrdo_st_b : in  STD_LOGIC_VECTOR(2 downto 0);	
		fsminstseq_st : in  STD_LOGIC_VECTOR(4 downto 0);	
		fsminstloop_st : in  STD_LOGIC_VECTOR(4 downto 0);
		fsmtsttrg_st : in  STD_LOGIC_VECTOR(1 downto 0);
		fsmttctrg_st : in  STD_LOGIC_VECTOR(3 downto 0);
		fsmev_st : in  STD_LOGIC_VECTOR(2 downto 0);	
		fsminsseq_st : in  STD_LOGIC_VECTOR(2 downto 0);		
		fsmru_st : in  STD_LOGIC_VECTOR(4 downto 0);		
		fsmdamain_st : in  STD_LOGIC_VECTOR(4 downto 0);	
		fsmrdcdh_st : in  STD_LOGIC_VECTOR(4 downto 0);	
		fsmrdcdhsoft_st : in  STD_LOGIC_VECTOR(4 downto 0);	
		fsmrddm_st : in  STD_LOGIC_VECTOR(4 downto 0);	
		fsmrdpar_st : in  STD_LOGIC_VECTOR(4 downto 0);	
		fsmrdrm_st : in  STD_LOGIC_VECTOR(3 downto 0)
		);
	END COMPONENT;
		
	COMPONENT ABS_CNT
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		clr : IN std_logic;
		dstb_a : IN std_logic;
		dstb_b : IN std_logic;
		trsf_a : IN std_logic;
		trsf_b : IN std_logic;
		ack_a : IN std_logic;
		ack_b : IN std_logic;
		cstb_a : IN std_logic;
		cstb_b : IN std_logic;          
		dstba_cnt : OUT std_logic_vector(19 downto 0);
		dstbb_cnt : OUT std_logic_vector(19 downto 0);
		trsfa_cnt : OUT std_logic_vector(11 downto 0);
		trsfb_cnt : OUT std_logic_vector(11 downto 0);
		acka_cnt : OUT std_logic_vector(11 downto 0);
		ackb_cnt : OUT std_logic_vector(11 downto 0);
		cstba_cnt : OUT std_logic_vector(11 downto 0);
		cstbb_cnt : OUT std_logic_vector(11 downto 0)
		);
	END COMPONENT;
	
	signal rst				:std_logic;
	signal rcu_add_t  	  	: std_logic_vector (15 downto 0);	-- TEMP FOR RCU_ADD
	signal rcu_dat_t 		: std_logic_vector (31 downto 0);	-- TEMP FOR RCU_DAT
	signal rcu_we_t, siubg_t, dcsbg_t : std_logic;	-- TEMP FOR rcu_we, siubg, dcsbg
	
	SIGNAL dm_en		 	: std_logic;	-- "sel_r" GATE OR
	SIGNAL rcu_im_en_r,rcu_rolm_en_r,rcu_rm_en_r,rcu_reg_en,rcu_cmd_en : std_logic;
	SIGNAL rcu_ttcrx_en 	: std_logic;
	SIGNAL mem_en   	  	: std_logic_vector (6 downto 0);
	SIGNAL sel_r   	  		: std_logic_vector (7 downto 0);
	SIGNAL rcu_hlm_en_r   	: std_logic;
	SIGNAL rcu_dout   	  	: std_logic_vector (31 downto 0);
	SIGNAL dout   	  			: std_logic_vector (31 downto 0);	
		
	SIGNAL clr_reg_cnt   	: std_logic;	-- clear error register and counters
	SIGNAL clr_rdrx   		: std_logic;	-- clear error register and counters
	
	-- ALTRO bus signal counters
	SIGNAL	dstba_cnt		: std_logic_vector(19 downto 0);	-- number of dstb negedge for branch a  
	SIGNAL	dstbb_cnt		: std_logic_vector(19 downto 0);	-- number of dstb negedge for branch b
	SIGNAL	trsfa_cnt		: std_logic_vector(11 downto 0);	-- number of trsf posedge for branch a  
	SIGNAL	trsfb_cnt		: std_logic_vector(11 downto 0);	-- number of trsf posedge for branch b 
	SIGNAL	acka_cnt		: std_logic_vector(11 downto 0);	-- number of ack posedge for branch a  
	SIGNAL	ackb_cnt		: std_logic_vector(11 downto 0);	-- number of ack posedge for branch b 
	SIGNAL	cstba_cnt		: std_logic_vector(11 downto 0);	-- number of cstb posedge for branch a  
	SIGNAL	cstbb_cnt		: std_logic_vector(11 downto 0);	-- number of cstb posedge for branch b 
	
	SIGNAL rcu_busbsy		: std_logic_vector(1 downto 0);		-- RCU bus busy status regsiters and IS get failed instruction register
--	SIGNAL dcs_br_ib	 	: std_logic;	-- dcs_br logic with ins_bsy
--	SIGNAL siu_br_ib	 	: std_logic;	-- siu_br logic with ins_bsy
	                                                    
	SIGNAL scel_err_reg   	: std_logic_vector (5 downto 0);	-- contain error code information from "EVLEN_FSM" and "SC_EVLEN_FSM"
	SIGNAL rdrx_erreg   	: std_logic; 	-- RDYRX  error register, from RCU_REG
	SIGNAL d_siu_rst	   	: std_logic_vector(2 downto 0); 	-- debug SIU reset status
	
	
	SIGNAL chadd_err_cnt_r	   	: std_logic_vector(11 downto 0); 	-- Channel address mismatch counter.
	SIGNAL bl_err_cnt_r		   	: std_logic_vector(11 downto 0); 	-- Block length mismatch counter.
	SIGNAL blk_err_reg		   	: std_logic; 	-- Block length mismatch error.
	SIGNAL add_err_reg			: std_logic; 	-- Block length mismatch error.	
	
begin

	rst <= not rst_n;
	rcu_busbsy(0) <= ins_bsy;
	rcu_busbsy(1) <= is_fail_reg;
	
	bl_err_cnt <= bl_err_cnt_r;
	chadd_err_cnt <= chadd_err_cnt_r;
	
--	dcs_br_ib <= dcs_br and (not ins_bsy);
--	siu_br_ib <= siu_br and (not ins_bsy);
	
	clr_err_reg <= clr_reg_cnt;
	
	dcs_bg <= dcsbg_t;
	siu_bg <= siubg_t;
	rcu_add <= rcu_add_t;
	rcu_data_out <= rcu_dat_t;
	rcu_wen <= rcu_we_t;
	
	rcu_im_en <= rcu_im_en_r;
	rcu_rolm_en <= rcu_rolm_en_r;
	rcu_rm_en <= rcu_rm_en_r;
	rcu_hlm_en <= rcu_hlm_en_r;
	sel <= sel_r;
	
	dm_en <= sel_r(0) or sel_r(1) or sel_r(2) or sel_r(3) or sel_r(4) or sel_r(5) or sel_r(6) or sel_r(7);
	mem_en <= rcu_im_en_r&rcu_rolm_en_r&rcu_rm_en_r&rcu_ttcrx_en&rcu_hlm_en_r&rcu_reg_en&dm_en;
	process(mem_en,im_dout,rcu_rolm_out,rm_dout,rcu_hlm_out,dout,rcu_dm_out,rcu_ttcrx_dout)
	begin
		case mem_en is
			when "1000000" =>
				rcu_dout(31 downto 22) <= (others=>'0');
				rcu_dout(21 downto  0) <= im_dout;				
			when "0100000" =>
				rcu_dout(31 downto 12) <= (others=>'0');
				rcu_dout(11 downto  0) <= rcu_rolm_out;				
			when "0010000" =>
				rcu_dout(31 downto 25) <= (others=>'0');
				rcu_dout(24 downto  0) <= rm_dout;							
			when "0001000" =>
				rcu_dout(31 downto  0) <= rcu_ttcrx_dout;	
			when "0000100" =>
				rcu_dout(31 downto  0) <= rcu_hlm_out;				
			when "0000010" =>
				rcu_dout(31 downto  0) <= dout;				
			when "0000001" =>
				rcu_dout(31 downto 28) <= (others=>'0');
				rcu_dout(27 downto  0) <= rcu_dm_out;				
			when others =>
				rcu_dout <= (others=>'Z');
		end case;
	end process;

	process(rst,clk,exe_ev)
	begin
		if rst='1' or exe_ev='1' then
			scel_err_reg(5) <= '0';
		elsif clk' event and clk='1' then
			if err_scev='1' then
				scel_err_reg(5) <= '1';
			end if;
		end if;
	end process;
	
	process(rst,clk,exe_ev)
	begin
		if rst='1' or exe_ev='1'then
			scel_err_reg(4 downto 0) <= (others=>'0');
		elsif clk' event and clk='1' then
			case err_code is
				when "001" =>
					scel_err_reg(0) <= '1';
				when "010" =>
					scel_err_reg(1) <= '1';
				when "011" =>
					scel_err_reg(2) <= '1';
				when "100" =>
					scel_err_reg(3) <= '1';
				when "101" =>
					scel_err_reg(4) <= '1';		
				when others =>
			end case;
		end if;
	end process;
		
	--------------U1_MUXARBIT--------------
	U1_MUXARBIT : MUX_ARBIT PORT MAP(
		dcs_din => dcs_busBdata_out,
		siu_din => siu_data_out,
		dcs_add => dcs_busBadd,
		siu_add => siu_add,
		dcs_we => dcs_we,
		siu_we => siu_we,
		rcu_dout => rcu_dout,
		dcs_dout => dcs_busBdata_in,
		siu_dout => siu_data_in,
		rcu_add => rcu_add_t,
		rcu_data => rcu_dat_t,
		rcu_we => rcu_we_t,
		siu_bg => siubg_t,
		dcs_bg => dcsbg_t
	);
	
	--------------U2_ARBIT--------------
	U2_ARBIT : ARBIT PORT MAP(
		siu_br => siu_br,
		siu_bi => siu_bi,
		dcs_br => dcs_br,
		dcs_bi => dcs_bi,
		clk => clk,
		rst => rst,
		siu_bg => siubg_t,
		dcs_bg => dcsbg_t
	);
	
	--------------U3_ADDRDECODER--------------
	U3_ADDRDECODER : ADDR_DECODER PORT MAP(
		rcu_add => rcu_add_t(15 downto 5),
		rcu_im_en => rcu_im_en_r,
		rcu_rolm_en => rcu_rolm_en_r,
		rcu_rm_en => rcu_rm_en_r,
		rcu_ttcrx_en => rcu_ttcrx_en,
		rcu_hlm_en => rcu_hlm_en_r,
		rcu_reg_en => rcu_reg_en,
		rcu_cmd_en => rcu_cmd_en,
		sel => sel_r
	);

	--------------U4_CMDDECODER--------------
	U4_CMDDECODER : CMD_DECODER PORT MAP(
		clk => clk,
		rst => rst,
		en => rcu_cmd_en,
		we => rcu_we_t,
		addr => rcu_add_t(4 downto 0),
		cfg_fec => cfg_fec,
		exe_seq => exe_seq,
		abrtseq => abrtseq,
		arm_syn => arm_syn,
		swt_trg => swt_trg,
		clr_err_reg => clr_reg_cnt,
		clr_rdrx_reg => clr_rdrx,
		clr_trg_cnt => clr_trg_cnt
	);

	--------------U5_RCUREG--------------
	U5_RCUREG : RCU_REG PORT MAP(		
		
		meb_full => meb_full,
		en => rcu_reg_en,
		we => rcu_we_t,
		addr => rcu_add_t(7 downto 0),
		din => rcu_dat_t,
		fecerr_a => fecerr_a,
		fecerr_b => fecerr_b,
		data_ready	=> data_ready,
		dm_sel	=> dm_sel,
		da_bsy => da_bsy,
		rd_rx => rd_rx,
		clr_rdrx_reg => clr_rdrx,
		scel_err_reg => scel_err_reg,
		blk_err_reg => blk_err_reg,
		add_err_reg => add_err_reg,
		rcu_busbsy => rcu_busbsy,
		meb_st_cnt => meb_st_cnt,
		swt_trg_cnt => swt_trg_cnt,
	--	aux_trg_cnt => st_wrd_cnt,
		aux_trg_cnt => aux_trg_cnt,
		ttc_L2a_cnt => ttc_L2a_cnt,
		ttc_L2r_cnt => ttc_L2r_cnt,
		ttc_L1r_cnt => ttc_L1r_cnt,
		dout => dout,
		fec_act_list => fec_act_list,
		inst_err_chk => inst_err_chk,
		prg_dly => prg_dly,
		samp_frec => samp_frec,
		trg_source => trg_source,
		trg_mode => trg_mode,
		trg_prg_dly => trg_prg_dly,
		en_chkrdyrx_n => en_chkrdyrx_n,
		sprs_rdo => sprs_rdo,
		skp_emp_ch => skp_emp_ch,
		mask_ch_add => mask_ch_add,
		mask_blk_len => mask_blk_len,
		es_on => es_on,
		meb_mode => meb_mode,
		rdo_cfg1 => rdo_cfg1,
		rdo_cfg2 => rdo_cfg2,
		rcu_version => rcu_version,
		rcu_id => rcu_id,
		bp_version => bp_version,
		cstb_a => cstb_a,
		ack_a  => ack_a,
		trsf_a => trsf_a,
		cstb_b => cstb_b,
		ack_b  => ack_b,
		trsf_b => trsf_b,
		L1 => L1,
		L2 => L2,
		altro_err_a	=>	altro_err_a,
		altro_err_b		=>	altro_err_b,
		dstba_cnt => dstba_cnt,
		dstbb_cnt => dstbb_cnt,
		trsfa_cnt => trsfa_cnt,
		trsfb_cnt => trsfb_cnt,
		acka_cnt => acka_cnt,
		ackb_cnt => ackb_cnt,
		cstba_cnt => cstba_cnt,
		cstbb_cnt => cstbb_cnt,
		dstb_numA => dstb_numA,
		dstb_numB => dstb_numB,
		fsmwr_st_b => fsmwr_st_a,		        	        
		fsmrd_st_a => fsmrd_st_b,
		fsmwr_st_a => fsmwr_st_a,		        	        
		fsmrd_st_b => fsmrd_st_b,		        
		fsmevrdoctrl_st => fsmevrdoctrl_st,	
		fsmselbr_st_a => fsmselbr_st_a,	  
		fsmselbr_st_b => fsmselbr_st_b,	  
		fsmevlen_st => fsmevlen_st,		      
		fsmscevlen_st => fsmscevlen_st,	  
		fsmrdo_st_a => fsmrdo_st_a,		      
		fsmrdo_st_b => fsmrdo_st_b,
		fsminstseq_st => fsminstseq_st,
		fsminstloop_st => fsminstloop_st,		
		fsmtsttrg_st => fsmtsttrg_st,
		fsmttctrg_st => fsmttctrg_st,
		fsmru_st => fsmru_st,
		fsmev_st => fsmev_st,
		fsminsseq_st => fsminsseq_st,
		fsmdamain_st => fsmdamain_st,
		fsmrdcdh_st => fsmrdcdh_st,
		fsmrdcdhsoft_st => fsmrdcdhsoft_st,
		fsmrddm_st => fsmrddm_st,
		fsmrdpar_st => fsmrdpar_st,
		fsmrdrm_st => fsmrdrm_st,
		rdrx_erreg => rdrx_erreg,
		rdo_err => rdo_err,
		bl_err_cnt => bl_err_cnt_r,
		chadd_err_cnt => chadd_err_cnt_r,
		FEC_AL_fsc => FEC_AL_fsc,
		we_FEC_AL_fsc => we_FEC_AL_fsc,
		clk => clk,
		rst => rst
	);

	--------------U6_ABSCNT--------------
	U6_ABSCNT : ABS_CNT PORT MAP(
		clk => clk,
		rst => rst,
		clr => clr_reg_cnt,
		dstb_a => dstb_a,
		dstb_b => dstb_b,
		trsf_a => trsf_a,
		trsf_b => trsf_b,
		ack_a => ack_a,
		ack_b => ack_b,
		cstb_a => cstb_a,
		cstb_b => cstb_b,
		dstba_cnt => dstba_cnt,
		dstbb_cnt => dstbb_cnt,
		trsfa_cnt => trsfa_cnt,
		trsfb_cnt => trsfb_cnt,
		acka_cnt => acka_cnt,
		ackb_cnt => ackb_cnt,
		cstba_cnt => cstba_cnt,
		cstbb_cnt => cstbb_cnt
	);
	
	--------------U7_BLERRCNT--------------
	U7_BLERRCNT : process(rst,clk,exe_ev)
	begin
		if rst='1' or exe_ev='1' then
			bl_err_cnt_r <= (others=>'0');
			blk_err_reg <= '0';
		elsif clk'event and clk='1' then
			if en_blerr_cnt='1' then
				bl_err_cnt_r <= bl_err_cnt_r + 1;
				blk_err_reg <= '1';
			end if;
		end if;
	end process;
		
	--------------U8_CHADDERRCNT--------------
	U8_CHADDERRCNT : process(rst,clk,exe_ev)
	begin
		if rst='1' or exe_ev='1' then
			chadd_err_cnt_r <= (others=>'0');
			add_err_reg <= '0';
		elsif clk'event and clk='1' then
			if en_chadderr_cnt='1' then
				chadd_err_cnt_r <= chadd_err_cnt_r + 1;
				add_err_reg <= '1';
			end if;
		end if;
	end process;
end;

